#include "Fl_DragBar.H"

#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Window.H>

#include <stdlib.h>
#include <string.h>

Fl_DragBar::Fl_DragBar(int x, int y, int w, int h, const char *l) : Fl_Widget(x, y, w, h, l) 
{
	mx_ = -1;
	my_ = -1;
}

void Fl_DragBar::draw() 
{
	draw_box();

  // full-window draggable background w/o decoration
  Fl_Window *win = window();
  if(win && (win->w() == w()) && (win->h() == h())) return;

  Fl_Color light = fl_lighter(color());
  Fl_Color dark = fl_darker(color());

	int x1 = x()+4;
	int y1 = y()+4;
	int x2 = x()+w()-4;
	int y2 = y()+h()-4;

  int horiz = (w() > h());

  if (horiz)
	{
		for (int yy=y1; yy<y2; yy+=3)
		{
			fl_color(light);
			fl_line(x1, yy, x2, yy);
			fl_color(dark);
			fl_line(x1, yy+1, x2, yy+1);
		}
  }
	else
	{	
		for (int xx=x1; xx<x2; xx+=3)
		{
			fl_color(light);
			fl_line(xx, y1, xx, y2);
      fl_color(dark);
			fl_line(xx+1, y1, xx+1, y2);
		}
	}

	const char *l = label();

  if(l && l[0] && horiz)
  {
		if(box())
    {
      if (l[0] != ' ') // pad the label
      {
        char *tmp = (char*)malloc(strlen(l) + 3);
        strcpy(tmp, " ");
        strcat(tmp, l);
        strcat(tmp, " ");
        copy_label(tmp);
        free(tmp);
      }

      fl_font(labelfont(), labelsize());
      int ww = (int)fl_width(label());
      int hh = h() - Fl::box_dh(box());

      y1 = y() + ((h() - hh) / 2);

      if (align() & FL_ALIGN_RIGHT)
		  {
			  x1 = x2 - ww;
		  }
		  else if (!(align() & FL_ALIGN_LEFT)) // FL_ALIGN_CENTER
		  {
			  x1 = x() + ((w() - ww) / 2);
		  }

      fl_rectf(x1, y1, ww, hh, color());
    }

		draw_label();
	}
}

int Fl_DragBar::handle(int e) 
{ 
  Fl_Group *grp = parent();
  Fl_Window *win = window();
  int mx,my,dx,dy;

  int windrag = (grp == (Fl_Group*)win);

	if (windrag)
	{
		mx = Fl::event_x_root(); 
		my = Fl::event_y_root();
	}
	else
	{ 
		mx = Fl::event_x();
		my = Fl::event_y();
	}
	dx = mx - mx_;
	dy = my - my_;
	mx_ = mx;
	my_ = my;

	switch(e)
	{
	case FL_PUSH:
		fl_cursor(FL_CURSOR_MOVE);
		return 1;

	case FL_RELEASE:
		fl_cursor(FL_CURSOR_DEFAULT);
		do_callback();
		return 1;

  case FL_DRAG:
		if (dx || dy)
		{
      grp->position(grp->x()+dx, grp->y()+dy);
      if (grp->parent()) grp = grp->parent();
			grp->redraw();		
		}
		return 1;
		
	case FL_LEAVE:
		fl_cursor(FL_CURSOR_DEFAULT);
		return 1;
  }

  return Fl_Widget::handle(e);
}

