#include "Fl_Multi_Widget.H"

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/fl_draw.H>

#include <string.h>

// warning: this widget assumes that if there is a callback, then userdata is a string

// dangerous, but the fl_autorun usage is safe-ish (as long as user_data_ is always a string)
#define looks_like_uri(s) (s && (!strncmp(s, "mailto:", 7) || strstr(s, "://")))

Fl_Multi_Widget::Fl_Multi_Widget(int x, int y, int w, int h, const char *l): Fl_Button(x, y, w, h, l) 
{
	texture_ = 0;
	picture_ = 0;
}

void Fl_Multi_Widget::draw() 
{
  // Fl_Button::draw() verbatim
  if (type() == FL_HIDDEN_BUTTON) return;
  Fl_Color col = value() ? selection_color() : color();
  draw_box(value() ? (down_box()?down_box():fl_down(box())) : box(), col);

  // add texture and picture
  int btex = texture_ && texture_->count();
  int bpic = picture_ && picture_->count();

  if (btex || bpic)
  {
    int xx = x() + Fl::box_dx(box());
    int yy = y() + Fl::box_dy(box());
    int ww = w() - Fl::box_dw(box());
    int hh = h() - Fl::box_dh(box());
    
    int xpos, ypos;

    fl_push_clip(xx, yy, ww, hh);

    if (btex)
    {
      for (ypos=0; ypos<hh; ypos+= texture_->h())
      {
        for (xpos=0; xpos<ww; xpos+= texture_->w())
        {
          texture_->draw(xx + xpos, yy + ypos);
        }
      }
    }

    if (bpic)
    {
      xpos = (ww - picture_->w()) / 2;
      ypos = (hh - picture_->h()) / 2;
      picture_->draw(xx + xpos, yy + ypos);
    }

    fl_pop_clip();
  }

  // Fl_Button::draw() verbatim
  if (labeltype() == FL_NORMAL_LABEL && value()) {
    Fl_Color c = labelcolor();
    labelcolor(fl_contrast(c, col));
    draw_label();
    labelcolor(c);
  } else draw_label();
  if (Fl::focus() == this) draw_focus();
}

int Fl_Multi_Widget::handle(int e) 
{
  if (callback() != Fl_Widget::default_callback)
  {
    int uri = looks_like_uri((char*)user_data());

    int floating = (FL_NO_BOX == box());

    if (uri || floating)
    {
      if (FL_ENTER == e) 
      {
        if(uri) window()->cursor(FL_CURSOR_HAND); 
        labelcolor_ = labelcolor();
        labelcolor(selection_color());
        redraw();
        if(floating) window()->redraw();
        return 1;
      }
      else if (FL_LEAVE == e) 
      {
        if(uri) window()->cursor(FL_CURSOR_DEFAULT); 
        labelcolor(labelcolor_);
        redraw(); 
        if(floating) window()->redraw();
        return 1;
      } 
    }

    return Fl_Button::handle(e);
  }
  else
  {
    return (e == FL_ENTER || e == FL_LEAVE)? 1 : 0;
  }
}

