#include "parse.h"
#include <ctype.h>
#include <string.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>


typedef struct vn_pair { int v; char *n; } VN_PAIR;


#define FL_DEFAULT "FL_DEFAULT"


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern size_t strltrim(char *s)
{
  size_t i, j, len;
  
  len = strlen(s);

  for(i=0; i<len; i++)
  {
    if(!isspace(s[i])) break;
  }
  
  if(i == 0) return len;
  
  for(j=0; i<len; i++,j++)
  {
    s[j] = s[i];
  }

  s[j] = '\0';
  return j;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern size_t strrtrim(char *s)
{
  size_t i, len;
  
  len = strlen(s);

  for(i=len; i>0; i--)
  {
    if(!isspace(s[i-1])) break;
  }

  s[i] = '\0';
  return i;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern size_t strtrim(char *s)
{
  strltrim(s);
  return strrtrim(s);
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


// limited, for now
//
extern size_t compose_escape_sequences(char *s)
{
  size_t i, j;

  for(i=0,j=0; s[i]; i++,j++)
  {
    if('\\' == s[i])
    {
      if('\\' == s[i+1])
      {
        s[j] = '\\';
        i++;
      }
      else if('n' == s[i+1])
      {
        s[j] = '\n';
        i++;
      }
      else if('r' == s[i+1])
      {
        s[j] = '\r';
        i++;
      }
      else if('t' == s[i+1])
      {
        s[j] = '\t';
        i++;
      }
      else s[j] = s[i];
    }
    else s[j] = s[i];
  }

  s[j] = '\0';
  return j;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern int readln(FILE *fp, char key[BUFSIZE], char arg[BUFSIZE])
{
  char buf[BUFSIZE];

  if(!fgets(buf, BUFSIZE, fp)) return 0;

  key[0] = arg[0] = 0;
  sscanf(buf, "%s %[^\n]", key, arg);
  strtrim(arg);
  compose_escape_sequences(arg);
  return 1;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern int parse_widget(const char *arg, int *x, int *y, int *w, int *h, char l[BUFSIZE])
{
  l[0] = 0;
  if(sscanf(arg, "%d %d %d %d %[^\0]", x, y, w, h, l) < 4) return 0;

  int xx, yy, ww, hh;
  Fl_Group *grp = Fl_Group::current();

  if(grp)
  {
    xx = grp->x();
    yy = grp->y();
    ww = grp->w();
    hh = grp->h();
  }
  else
  {
    xx = Fl::x();
    yy = Fl::y();
    ww = Fl::w();
    hh = Fl::h();
  }

  if(*w < 0 || *w > ww) *w = ww;
  if(*h < 0 || *h > hh) *h = hh;

  if(*x < 0) *x = (ww - *w)/2;
  else if(*x+*w > xx+ww) *x = 0;

  if(*y < 0) *y = (hh - *h)/2;
  else if(*y+*h > yy+hh) *y = 0;

  return 1;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern Fl_Labeltype parse_labeltype(const char *arg, Fl_Labeltype def)
{
  static const char *types[] = 
  {
    "FL_NORMAL_LABEL",
    "FL_NO_LABEL",
    "FL_SHADOW_LABEL",
    "FL_ENGRAVED_LABEL",
    "FL_EMBOSSED_LABEL",
    0
  };

  if(!strcmp(arg, FL_DEFAULT)) return def;

  for(int i=0; types[i]; i++)
  {
    if(!strcmp(arg, types[i]))
    {
      return (Fl_Labeltype)i;
    }
  }

  return def;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern Fl_Font parse_labelfont(const char *arg, Fl_Font def)
{
  static const char *types[] = 
  {
    "FL_HELVETICA",
    "FL_HELVETICA_BOLD",
    "FL_HELVETICA_ITALIC",
    "FL_HELVETICA_BOLD_ITALIC",
    "FL_COURIER",
    "FL_COURIER_BOLD",
    "FL_COURIER_ITALIC",
    "FL_COURIER_BOLD_ITALIC",
    "FL_TIMES",
    "FL_TIMES_BOLD",
    "FL_TIMES_ITALIC",
    "FL_TIMES_BOLD_ITALIC",
    "FL_SYMBOL",
    "FL_SCREEN",
    "FL_SCREEN_BOLD",
    "FL_ZAPF_DINGBATS",
    0
  };

  if(!strcmp(arg, FL_DEFAULT)) return def;

  for(int i=0; types[i]; i++)
  {
    if(!strcmp(arg, types[i]))
    {
      return (Fl_Font)i;
    }
  }

  return def;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern Fl_Boxtype parse_box(const char *arg, Fl_Boxtype def)
{
  static const char *types[] = 
  {
    "FL_NO_BOX", "FL_FLAT_BOX", 
    "FL_UP_BOX", "FL_DOWN_BOX", 
    "FL_UP_FRAME", "FL_DOWN_FRAME", 
    "FL_THIN_UP_BOX", "FL_THIN_DOWN_BOX", 
    "FL_THIN_UP_FRAME", "FL_THIN_DOWN_FRAME", 
    "FL_ENGRAVED_BOX", "FL_EMBOSSED_BOX", 
    "FL_ENGRAVED_FRAME", "FL_EMBOSSED_FRAME", 
    "FL_BORDER_BOX", "FL_SHADOW_BOX", 
    "FL_BORDER_FRAME", "FL_SHADOW_FRAME", 
    "FL_ROUNDED_BOX", "FL_RSHADOW_BOX", 
    "FL_ROUNDED_FRAME", "FL_RFLAT_BOX", 
    "FL_ROUND_UP_BOX", "FL_ROUND_DOWN_BOX", 
    "FL_DIAMOND_UP_BOX", "FL_DIAMOND_DOWN_BOX", 
    "FL_OVAL_BOX", "FL_OSHADOW_BOX", 
    "FL_OVAL_FRAME", "FL_OFLAT_BOX", 
    "FL_PLASTIC_UP_BOX", "FL_PLASTIC_DOWN_BOX", 
    "FL_PLASTIC_UP_FRAME", "FL_PLASTIC_DOWN_FRAME", 
    "FL_PLASTIC_THIN_UP_BOX", "FL_PLASTIC_THIN_DOWN_BOX", 
    "FL_PLASTIC_ROUND_UP_BOX", "FL_PLASTIC_ROUND_DOWN_BOX", 
    "FL_GTK_UP_BOX", "FL_GTK_DOWN_BOX", 
    "FL_GTK_UP_FRAME", "FL_GTK_DOWN_FRAME", 
    "FL_GTK_THIN_UP_BOX", "FL_GTK_THIN_DOWN_BOX", 
    "FL_GTK_THIN_UP_FRAME", "FL_GTK_THIN_DOWN_FRAME", 
    "FL_GTK_ROUND_UP_BOX", "FL_GTK_ROUND_DOWN_BOX", 
    0
  };

  if(!strcmp(arg, FL_DEFAULT)) return def;

  for(int i=0; types[i]; i++)
  {
    if(!strcmp(arg, types[i]))
    {
      return (Fl_Boxtype)i;
    }
  }

  return def;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern Fl_Align parse_align(const char *arg, Fl_Align def)
{
  static const vn_pair types[] = 
  {
    { 0,   "FL_ALIGN_CENTER" }, 
    { 1,   "FL_ALIGN_TOP" }, 
    { 2,   "FL_ALIGN_BOTTOM" }, 
    { 4,   "FL_ALIGN_LEFT" }, 
    { 8,   "FL_ALIGN_RIGHT" }, 
    { 16,  "FL_ALIGN_INSIDE" }, 
    { 32,  "FL_ALIGN_TEXT_OVER_IMAGE" }, 
    { 0,   "FL_ALIGN_IMAGE_OVER_TEXT" }, 
    { 64,  "FL_ALIGN_CLIP" }, 
    { 128, "FL_ALIGN_WRAP" }, 
    { 5,   "FL_ALIGN_TOP_LEFT" }, 
    { 9,   "FL_ALIGN_TOP_RIGHT" }, 
    { 6,   "FL_ALIGN_BOTTOM_LEFT" }, 
    { 10,  "FL_ALIGN_BOTTOM_RIGHT" }, 
    { 5,   "FL_ALIGN_LEFT_TOP" }, 
    { 9,   "FL_ALIGN_RIGHT_TOP" }, 
    { 6,   "FL_ALIGN_LEFT_BOTTOM" }, 
    { 10,  "FL_ALIGN_RIGHT_BOTTOM" }, 
    { 0,   "FL_ALIGN_NOWRAP" }, 
    { -1,  "" }
  };

  if(!strcmp(arg, FL_DEFAULT)) return def;

  int ok = 0;
  int align = 0;

  char buf[BUFSIZE];
  strcpy(buf, arg);
  
  char *a = strtok(buf, "|");
  while(a)
  {
    strtrim(a);

    for(int i=0; types[i].v != -1; i++)
    {
      if(!strcmp(a, types[i].n))
      {
        align |= types[i].v;
        ok = 1;
        break;
      }
    }

    a = strtok(0, "|");
  }

  return (ok)? (Fl_Align)align : def;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern Fl_Color parse_color(const char *arg, Fl_Color def)
{
  static const vn_pair types[] = 
  {
    { FL_BACKGROUND_COLOR, "FL_BACKGROUND_COLOR" }, 
    { FL_SELECTION_COLOR,  "FL_SELECTION_COLOR" }, 
    { -1, "" }
  };

  if(!strcmp(arg, FL_DEFAULT)) return def;

  for(int i=0; types[i].v != -1; i++)
  {
    if(!strcmp(arg, types[i].n))
    {
      return (Fl_Color)types[i].v;
    }
  }

  int r, g, b;
  if(3 == sscanf(arg, "%d %d %d", &r, &g, &b)) 
  {
    return fl_rgb_color(r, g, b);
  }

  return def;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern uchar parse_labelsize(const char *arg, uchar def)
{
  int n;

  if(!strcmp(arg, FL_DEFAULT)) return def;

  if((1 == sscanf(arg, "%d", &n)) && (n>=6 && n<=255)) return (uchar)n;

  return def;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


extern const char *parse_scheme(const char *arg, const char *def)
{
  if(!strcmp(arg, FL_DEFAULT)) return def;

  if(!strcmp(arg, "NULL"))    return NULL;
  if(!strcmp(arg, "none"))    return arg;
  if(!strcmp(arg, "plastic")) return arg;
  if(!strcmp(arg, "gtk+"))    return arg;

  return def;
}


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

